// ----------------------------------------------------------------------------
// * Raima Database Manager
// *
// * Copyright (c) 2012 Raima Inc.,  All rights reserved.
// *
// * Use of this software, whether in source code format, or in executable,
// * binary object code form, is governed by the Raima LICENSE which
// * is fully described in the LICENSE.TXT file, included within this
// * distribution of files.
// * ----------------------------------------------------------------------------

#include <iostream>

// rdm.h is required in all files using the RDM Database
#include "rdm.h"

// obj11_db.h was generated from the DDL file obj11_db.ddl.
#include "obj11_db.h"

// obj11_db_api.h was generated from the DDL file obj11_db.ddl. with 
// the -cpp option
#include "obj11_db_api.h"

// Name space for the RDM C++ interface
using namespace RDM_CPP;

// EXAMPLE - obj11 for C++
// Each time you run this example:
//   1) The database is initialized (all existing contents removed)
//   2) A transaction is started
//   3) Several new offices are added to the database
//   4) The transaction is committed
//   5) Data is queried from the database


// The DDL (Database Definition Language) is in obj11_db.ddl. The database
// schema is embedded into the application and the database will be created
// the first time the application is run.  Subsequent runs will use the now
// existing database.

// The following RDM C++ methods are illustrated:
//      Db->Open
//      Db->Initialize
//      Db->BeginUpdate
//      Db->End
//      Db->BeginRead
//      Db->New_recordWithFieldValues
//      Db->Get_recordsByKey
//      Cursor->AfterLast
//      Cursor->GetFieldValues
//      Cursor++
// They are fully documented in http://docs.raima.com/rdm/.

// Each of these methods can throw and an exception of type rdm_exception

// The actual database is stored in a directory named 'obj11_db' in this
// project directory.

class obj11
{
public:
    Db_obj11_db    db;

    void InsertData () throw (rdm_exception&);
    void DisplayOffices (Db_obj11_db&) throw (rdm_exception&);
    void Initialize () throw (rdm_exception&);
    void Cleanup () throw (rdm_exception&);
};

void obj11::Initialize()
{
    // Open the North American database and initialize it
    try
    {
        db = db.Open("obj11_db");
        db.Initialize();
    }
    catch (rdm_exception& e)
    {
        cerr << "Error " << e.error() << " (" << e.what() << ") opening/initializing obj11_db database" << endl;
        throw (e);
        return;
    }
    return;
}

void obj11::InsertData()
{
    struct office office_rec;
    recordType rtypes[]= {Cursor_office::Type (), Cursor::NoMoreTypes()};
    
    try
    {
        // Start a transaction
        db.BeginUpdate(rtypes);

        // Create offices in the database
        strcpy(office_rec.name, "Seattle");
        db.New_office_recordWithFieldValues(&office_rec);

        strcpy(office_rec.name, "Boise");
        db.New_office_recordWithFieldValues(&office_rec);

        strcpy(office_rec.name, "San Francisco");
        db.New_office_recordWithFieldValues(&office_rec);

        strcpy(office_rec.name, "Dallas");
        db.New_office_recordWithFieldValues(&office_rec);

        strcpy(office_rec.name, "Paris");
        db.New_office_recordWithFieldValues(&office_rec);

        strcpy(office_rec.name, "London");
        db.New_office_recordWithFieldValues(&office_rec);

        strcpy(office_rec.name, "Dublin");
        db.New_office_recordWithFieldValues(&office_rec);

        strcpy(office_rec.name, "Zurich");
        db.New_office_recordWithFieldValues(&office_rec);

        strcpy(office_rec.name, "Madrid");
        db.New_office_recordWithFieldValues(&office_rec);
    }
    catch (rdm_exception& e)
    {
        db.Abort();
        cerr << "Error " << e.error() << " (" << e.what() << ") adding records" << endl;
        throw (e);
        return;
    }

    // Commit the transaction
    db.End();

    return;
}
    
void obj11::DisplayOffices(
    Db_obj11_db& db)
{
    struct office office_rec;
    Cursor_office office_cursor;
    recordType rtypes[]= {Cursor_office::Type (), Cursor::NoMoreTypes()};

    try
    {
        // Get read lock on the office record
        db.BeginRead(rtypes);

        // Loop through each office record and display it
        office_cursor = db.Get_office_recordsBy_name();
        while (!office_cursor.AfterLast())
        {
            office_cursor.GetFieldValues(&office_rec);
            cout << '\t' << office_rec.name << endl;
            office_cursor++;
        }
        db.End();
    }
    catch (rdm_exception& e)
    {
        db.End();
        cerr << "Error " << e.error() << " (" << e.what() << ") reading records" << endl;
    }   
}

void obj11::Cleanup()
{
    return;
}

int32_t main(void)
{
    obj11 obj;

    try
    {
        obj.Initialize();
        obj.InsertData();

        cout << endl << "All Worldwide Offices" << endl;
        obj.DisplayOffices(obj.db);
        
        obj.Cleanup();
    }
    catch (rdm_exception& e)
    {
        cerr << "Error " << e.error() << " (" << e.what() << ")" << endl;
        return EXIT_FAILURE;
    }
    
    cout << endl << "Press <Enter> to continue . . .";
    getchar();
    return EXIT_SUCCESS;
}
